from starlette.applications import Starlette
from starlette.responses import JSONResponse
from starlette.routing import Route
from starlette.testclient import TestClient

from mcp_auth_middleware.middleware import AuthUser, JWKSAuthMiddleware, ScopeDefinition, get_user

SCOPES = [
    {"scope": "name"},
    {"scope": "email"},
]


class FakeVerifier:
    def __init__(self, claims: dict | None = None, jwks: dict | None = None) -> None:
        self._claims = claims
        self._jwks = jwks or {"keys": [{"kty": "RSA"}]}
        self.is_configured = True

    async def verify_token(self, token: str) -> dict | None:
        return self._claims

    def get_jwks(self) -> dict:
        return self._jwks


def build_app(
    verifier: FakeVerifier,
    scopes: list[dict[str, str]] | None = None,
    jwks_path: str = "/.well-known/jwks.json",
    scopes_path: str = "/.well-known/fhswf-scopes",
) -> Starlette:
    async def me(request):
        return JSONResponse(dict(get_user()))

    routes = [Route("/me", me)]
    app = Starlette(routes=routes)
    app.add_middleware(
        JWKSAuthMiddleware,
        verifier=verifier,
        scopes=scopes or SCOPES,
        jwks_path=jwks_path,
        scopes_path=scopes_path,
    )
    return app


def test_auth_user_attribute_access() -> None:
    user = AuthUser({"email": "dev@example.com"})
    assert user.email == "dev@example.com"
    assert user.missing is None


def test_middleware_requires_at_least_one_scope() -> None:
    try:
        JWKSAuthMiddleware(Starlette(), verifier=FakeVerifier(), scopes=[])
    except ValueError as exc:
        assert "at least one configured scope" in str(exc)
    else:
        raise AssertionError("Expected ValueError for empty scopes")


def test_middleware_rejects_duplicate_scopes() -> None:
    try:
        JWKSAuthMiddleware(
            Starlette(),
            verifier=FakeVerifier(),
            scopes=[
                {
                    "scope": "email",
                    "description": "E-Mail-Adresse",
                    "description_en": "Email address",
                },
                ScopeDefinition(scope="email"),
            ],
        )
    except ValueError as exc:
        assert "Duplicate scope configured: email" == str(exc)
    else:
        raise AssertionError("Expected ValueError for duplicate scopes")


def test_middleware_serves_jwks() -> None:
    jwks = {"keys": [{"kty": "RSA", "kid": "kid1"}]}
    app = build_app(FakeVerifier(jwks=jwks))

    client = TestClient(app)
    response = client.get("/.well-known/jwks.json")

    assert response.status_code == 200
    assert response.json() == jwks


def test_middleware_serves_scopes_with_cors_headers() -> None:
    app = build_app(FakeVerifier())

    client = TestClient(app)
    response = client.get("/.well-known/fhswf-scopes")

    assert response.status_code == 200
    assert response.headers["content-type"].startswith("application/json")
    assert response.headers["access-control-allow-origin"] == "*"
    assert response.headers["access-control-allow-methods"] == "GET"
    assert response.json() == {"scopes_supported": SCOPES}


def test_middleware_sets_user_from_bearer_token() -> None:
    claims = {"sub": "123", "name": "Dev", "email": "dev@example.com"}
    app = build_app(FakeVerifier(claims=claims))

    client = TestClient(app)
    response = client.get("/me", headers={"Authorization": "Bearer token"})

    assert response.status_code == 200
    assert response.json() == claims
    assert dict(get_user()) == {}


def test_middleware_rejects_missing_bearer_token() -> None:
    app = build_app(FakeVerifier(claims={"name": "Dev", "email": "dev@example.com"}))

    client = TestClient(app)
    response = client.get("/me")

    assert response.status_code == 401
    assert response.headers["www-authenticate"] == "Bearer"
    assert response.json() == {"error": "invalid_token"}


def test_middleware_rejects_non_bearer_auth() -> None:
    app = build_app(FakeVerifier(claims={"name": "Dev", "email": "dev@example.com"}))

    client = TestClient(app)
    response = client.get("/me", headers={"Authorization": "Basic abc"})

    assert response.status_code == 401
    assert response.json() == {"error": "invalid_token"}


def test_middleware_handles_failed_verification() -> None:
    app = build_app(FakeVerifier(claims=None))

    client = TestClient(app)
    response = client.get("/me", headers={"Authorization": "Bearer bad"})

    assert response.status_code == 401
    assert response.json() == {"error": "invalid_token"}


def test_middleware_serves_scope_only_even_with_legacy_scope_config() -> None:
    app = build_app(
        FakeVerifier(),
        scopes=[
            {
                "scope": "email",
                "description": "E-Mail-Adresse",
                "description_en": "Email address",
            }
        ],
    )

    client = TestClient(app)
    response = client.get("/.well-known/fhswf-scopes")

    assert response.status_code == 200
    assert response.json() == {"scopes_supported": [{"scope": "email"}]}


def test_middleware_rejects_missing_scopes_with_scope_only_payload() -> None:
    app = build_app(FakeVerifier(claims={"name": "Dev"}))

    client = TestClient(app)
    response = client.get("/me", headers={"Authorization": "Bearer token"})

    assert response.status_code == 403
    assert response.json() == {
        "error": "missing_scopes",
        "missing": [{"scope": "email"}],
    }



#!/usr/bin/env python3
"""MCP Secrets CLI - Key management for JWE authentication."""

import argparse
import base64
import json
import os
import secrets
import shutil
import subprocess
import sys
from pathlib import Path

try:
    from jwcrypto import jwk
except ImportError:
    print("Error: 'jwcrypto' library is required.", file=sys.stderr)
    print("Please run: pip install jwcrypto", file=sys.stderr)
    sys.exit(1)

KEY_SIZE = 4096
KEY_FILES = ("mcp-private.json", "mcp-public.json")


def generate_keys(output_dir: Path) -> tuple[Path, Path]:
    """Generate an RSA JWKS key pair and write to output_dir."""
    output_dir.mkdir(parents=True, exist_ok=True)

    key = jwk.JWK.generate(kty="RSA", size=KEY_SIZE)
    key["kid"] = key.thumbprint()

    private_key = json.loads(key.export_private())
    public_key = json.loads(key.export_public())

    for exported_key in (private_key, public_key):
        exported_key["alg"] = "RSA-OAEP"
        exported_key["use"] = "enc"

    private_jwks = {"keys": [private_key]}
    public_jwks = {"keys": [public_key]}

    private_key_path = output_dir / "mcp-private.json"
    public_key_path = output_dir / "mcp-public.json"

    with open(private_key_path, "w") as f:
        json.dump(private_jwks, f, indent=2)

    try:
        os.chmod(private_key_path, 0o600)
    except OSError:
        pass

    with open(public_key_path, "w") as f:
        json.dump(public_jwks, f, indent=2)

    return private_key_path, public_key_path


def secure_delete(file_path: Path) -> None:
    """Securely delete a file using shred or overwrite fallback."""
    if not file_path.exists():
        return

    if shutil.which("shred"):
        subprocess.run(
            ["shred", "--remove", "--zero", "--iterations=3", str(file_path)],
            check=True,
        )
        return

    try:
        size = file_path.stat().st_size
        with open(file_path, "r+b") as f:
            f.write(secrets.token_bytes(size))
            f.flush()
            os.fsync(f.fileno())
    except OSError:
        pass

    file_path.unlink()


def clean_keys(key_dir: Path) -> list[str]:
    """Remove key files from key_dir. Returns list of removed paths."""
    removed = []
    for name in KEY_FILES:
        key_file = key_dir / name
        if not key_file.exists():
            continue

        if "private" in name:
            secure_delete(key_file)
        else:
            key_file.unlink()

        removed.append(str(key_file))
    return removed


def cmd_generate(args: argparse.Namespace) -> None:
    output_dir = args.output.resolve()
    private_key, public_key = generate_keys(output_dir)

    print(f"Keys generated (JWKS format):")
    print(f"  Private: {private_key}")
    print(f"  Public:  {public_key}")
    print(f"\nFor local development, add to .env:")
    print(f"  MCP_KEY_FILE_PATH={private_key}")
    print(f"\nAdd to .gitignore:")
    print(f"  .keys/")


def cmd_k8s(args: argparse.Namespace) -> None:
    output_dir = args.output.resolve()
    private_key, _ = generate_keys(output_dir)

    with open(private_key, "rb") as f:
        b64_key = base64.b64encode(f.read()).decode("utf-8")

    k8s_yaml = f"""\
apiVersion: v1
kind: Secret
metadata:
  name: {args.secret_name}
  namespace: {args.namespace}
type: Opaque
data:
  mcp_jwks: {b64_key}
"""
    print(k8s_yaml)
    print(f"# Apply: mcp-secrets k8s | kubectl apply -f -", file=sys.stderr)
    print(f"# Then clean local keys: mcp-secrets clean", file=sys.stderr)


def cmd_clean(args: argparse.Namespace) -> None:
    removed = clean_keys(args.output.resolve())
    if removed:
        print("Removed:\n  " + "\n  ".join(removed))
    else:
        print(f"No keys found in {args.output}")


def main() -> None:
    parser = argparse.ArgumentParser(
        prog="mcp-secrets", description="MCP Secrets - Key Management (JWKS)"
    )
    subs = parser.add_subparsers(dest="command", required=True)

    commands = [
        ("generate", "Generate RSA JWKS key pair", cmd_generate),
        ("k8s", "Generate keys and output Kubernetes secret YAML", cmd_k8s),
        ("clean", "Remove keys securely", cmd_clean),
    ]

    for name, help_text, handler in commands:
        p = subs.add_parser(name, help=help_text)
        p.add_argument(
            "-o",
            "--output",
            type=Path,
            default=Path(".keys"),
            help="Key directory (default: .keys)",
        )
        p.set_defaults(func=handler)

    k8s = subs.choices["k8s"]
    k8s.add_argument(
        "-n", "--namespace", default="default", help="Kubernetes namespace"
    )
    k8s.add_argument(
        "-s", "--secret-name", default="mcp-server-keys", help="Secret name"
    )

    args = parser.parse_args()
    args.func(args)


if __name__ == "__main__":
    main()



import asyncio
import json
import uuid
from pathlib import Path
from types import SimpleNamespace

import mcp_auth_middleware.verifier as verifier_module
from mcp_auth_middleware.verifier import JWETokenVerifier


def write_key_file(contents: dict | str) -> str:
    temp_dir = Path(__file__).resolve().parent.parent / ".test-tmp" / str(uuid.uuid4())
    temp_dir.mkdir(parents=True, exist_ok=True)
    path = Path(temp_dir) / "key.json"
    if isinstance(contents, str):
        path.write_text(contents)
    else:
        path.write_text(json.dumps(contents))
    return str(path)


def test_verifier_no_key_configured(monkeypatch) -> None:
    monkeypatch.delenv("MCP_KEY_FILE_PATH", raising=False)

    verifier = JWETokenVerifier()

    assert verifier.is_configured is False
    assert verifier.get_jwks() == {"keys": []}


def test_verifier_invalid_json(monkeypatch) -> None:
    monkeypatch.setenv("MCP_KEY_FILE_PATH", write_key_file("{bad-json"))

    verifier = JWETokenVerifier()

    assert verifier.is_configured is False


def test_verifier_loads_first_jwks_key_and_filters_public_fields(monkeypatch) -> None:
    jwks = {
        "keys": [
            {
                "kty": "RSA",
                "kid": "kid1",
                "alg": "RSA-OAEP",
                "use": "enc",
                "n": "n",
                "e": "e",
                "d": "private",
            }
        ]
    }
    monkeypatch.setenv("MCP_KEY_FILE_PATH", write_key_file(jwks))
    monkeypatch.setattr(verifier_module.jwk, "import_key", lambda raw_jwk: object())

    verifier = JWETokenVerifier()

    assert verifier.is_configured is True
    assert verifier.get_jwks() == {
        "keys": [
            {
                "kty": "RSA",
                "kid": "kid1",
                "alg": "RSA-OAEP",
                "use": "enc",
                "n": "n",
                "e": "e",
            }
        ]
    }


def test_verify_token_returns_claims(monkeypatch) -> None:
    monkeypatch.setenv("MCP_KEY_FILE_PATH", write_key_file({"kty": "RSA", "kid": "kid1"}))
    monkeypatch.setattr(verifier_module.jwk, "import_key", lambda raw_jwk: object())
    monkeypatch.setattr(verifier_module.time, "time", lambda: 1000)

    verifier = JWETokenVerifier()

    def fake_decrypt(token, key, algorithms=None):
        assert algorithms == verifier_module.ALLOWED_JWE_ALGORITHMS
        return SimpleNamespace(plaintext=b'{"data": {"sub": "123"}, "iat": 900, "exp": 1100}')

    monkeypatch.setattr(verifier_module.jwe, "decrypt_compact", fake_decrypt)

    assert asyncio.run(verifier.verify_token("token")) == {"sub": "123"}


def test_verify_token_rejects_missing_timestamps(monkeypatch) -> None:
    monkeypatch.setenv("MCP_KEY_FILE_PATH", write_key_file({"kty": "RSA", "kid": "kid1"}))
    monkeypatch.setattr(verifier_module.jwk, "import_key", lambda raw_jwk: object())
    monkeypatch.setattr(verifier_module.time, "time", lambda: 1000)

    verifier = JWETokenVerifier()

    def fake_decrypt(token, key, algorithms=None):
        return SimpleNamespace(plaintext=b'{"data": {"sub": "123"}}')

    monkeypatch.setattr(verifier_module.jwe, "decrypt_compact", fake_decrypt)

    assert asyncio.run(verifier.verify_token("token")) is None


def test_verify_token_rejects_future_iat(monkeypatch) -> None:
    monkeypatch.setenv("MCP_KEY_FILE_PATH", write_key_file({"kty": "RSA", "kid": "kid1"}))
    monkeypatch.setattr(verifier_module.jwk, "import_key", lambda raw_jwk: object())
    monkeypatch.setattr(verifier_module.time, "time", lambda: 1000)

    verifier = JWETokenVerifier()

    def fake_decrypt(token, key, algorithms=None):
        return SimpleNamespace(plaintext=b'{"data": {"sub": "123"}, "iat": 1001, "exp": 1100}')

    monkeypatch.setattr(verifier_module.jwe, "decrypt_compact", fake_decrypt)

    assert asyncio.run(verifier.verify_token("token")) is None


def test_verify_token_rejects_expired_token(monkeypatch) -> None:
    monkeypatch.setenv("MCP_KEY_FILE_PATH", write_key_file({"kty": "RSA", "kid": "kid1"}))
    monkeypatch.setattr(verifier_module.jwk, "import_key", lambda raw_jwk: object())
    monkeypatch.setattr(verifier_module.time, "time", lambda: 1000)

    verifier = JWETokenVerifier()

    def fake_decrypt(token, key, algorithms=None):
        return SimpleNamespace(plaintext=b'{"data": {"sub": "123"}, "iat": 900, "exp": 999}')

    monkeypatch.setattr(verifier_module.jwe, "decrypt_compact", fake_decrypt)

    assert asyncio.run(verifier.verify_token("token")) is None


def test_verify_token_invalid_payload(monkeypatch) -> None:
    monkeypatch.setenv("MCP_KEY_FILE_PATH", write_key_file({"kty": "RSA", "kid": "kid1"}))
    monkeypatch.setattr(verifier_module.jwk, "import_key", lambda raw_jwk: object())

    verifier = JWETokenVerifier()

    def fake_decrypt(token, key, algorithms=None):
        return SimpleNamespace(plaintext=b"not json")

    monkeypatch.setattr(verifier_module.jwe, "decrypt_compact", fake_decrypt)

    assert asyncio.run(verifier.verify_token("token")) is None


def test_verify_token_decrypt_exception(monkeypatch) -> None:
    monkeypatch.setenv("MCP_KEY_FILE_PATH", write_key_file({"kty": "RSA", "kid": "kid1"}))
    monkeypatch.setattr(verifier_module.jwk, "import_key", lambda raw_jwk: object())

    verifier = JWETokenVerifier()

    def fake_decrypt(token, key, algorithms=None):
        raise RuntimeError("boom")

    monkeypatch.setattr(verifier_module.jwe, "decrypt_compact", fake_decrypt)

    assert asyncio.run(verifier.verify_token("token")) is None



"""MCP Secrets - JWE authentication middleware for FastMCP/Starlette."""

from .middleware import AuthUser, JWKSAuthMiddleware, ScopeDefinition, get_user
from .verifier import JWETokenVerifier

__version__ = "0.1.0"
__all__ = ["JWETokenVerifier", "JWKSAuthMiddleware", "ScopeDefinition", "get_user", "AuthUser"]



"""JWE token verification and JWKS support."""
import json
import logging
import os
import time
from typing import Any

from joserfc import jwe, jwk

logger = logging.getLogger(__name__)

ALLOWED_JWE_ALGORITHMS = ["RSA-OAEP", "A256GCM"]


class JWETokenVerifier:
    """Verifies JWE tokens and exposes public key as JWKS."""

    def __init__(self) -> None:
        self._jwk = self._load_key_from_env()
        self._key = self._import_key(self._jwk) if self._jwk else None
        if self._jwk and self._key:
            kid = self._jwk.get("kid", "unknown")
            kty = self._jwk.get("kty", "unknown")
            logger.info("JWE key loaded successfully (kid=%s, kty=%s)", kid, kty)
        else:
            logger.warning(
                "No JWE key configured - token verification is disabled. "
                "Set MCP_KEY_FILE_PATH to enable authentication."
            )

    @property
    def is_configured(self) -> bool:
        """Whether a decryption key is available."""
        return self._key is not None

    @staticmethod
    def _load_key_from_env() -> dict | None:
        path = os.environ.get("MCP_KEY_FILE_PATH")
        if not path:
            logger.debug("MCP_KEY_FILE_PATH not set")
            return None

        logger.debug("Loading key from %s", path)
        try:
            with open(path) as f:
                data = json.load(f)
        except FileNotFoundError:
            logger.error("Key file not found: %s", path)
            return None
        except json.JSONDecodeError as e:
            logger.error("Key file is not valid JSON (%s): %s", path, e)
            return None

        if "keys" in data and isinstance(data["keys"], list):
            if not data["keys"]:
                logger.error("JWKS file contains empty 'keys' array: %s", path)
                return None
            logger.debug("Loaded first key from JWKS key set (%d keys total)", len(data["keys"]))
            return data["keys"][0]

        return data

    @staticmethod
    def _import_key(raw_jwk: dict[str, Any] | None) -> Any | None:
        if raw_jwk is None:
            return None

        try:
            return jwk.import_key(raw_jwk)
        except Exception as e:
            logger.error("Failed to import JWK: %s", e)
            return None

    async def verify_token(self, token: str) -> dict[str, Any] | None:
        if not self._key:
            logger.debug("Skipping token verification - no key configured")
            return None

        try:
            decrypted = jwe.decrypt_compact(token, self._key, algorithms=ALLOWED_JWE_ALGORITHMS)
            payload = json.loads(decrypted.plaintext)
            if not isinstance(payload, dict):
                logger.debug("Token payload is not a JSON object")
                return None
            if not self._timestamps_are_valid(payload):
                return None
            claims = payload.get("data", payload)
            if not isinstance(claims, dict):
                logger.debug("Token payload is not a JSON object")
                return None
            logger.debug("Token verified successfully")
            return claims
        except json.JSONDecodeError as e:
            logger.debug("Token decrypted but payload is not valid JSON: %s", e)
            return None
        except Exception as e:
            logger.debug("Token decryption detail: %s", e)
            return None

    @staticmethod
    def _timestamps_are_valid(payload: dict[str, Any]) -> bool:
        iat = payload.get("iat")
        exp = payload.get("exp")
        if not JWETokenVerifier._is_numeric_date(iat) or not JWETokenVerifier._is_numeric_date(exp):
            logger.debug("Token missing numeric iat/exp claims")
            return False

        now = time.time()
        if iat > now:
            logger.debug("Token iat is in the future")
            return False
        if exp <= iat:
            logger.debug("Token exp must be after iat")
            return False
        if exp <= now:
            logger.debug("Token has expired")
            return False

        return True

    @staticmethod
    def _is_numeric_date(value: Any) -> bool:
        return isinstance(value, int | float) and not isinstance(value, bool)

    def get_jwks(self) -> dict[str, Any]:
        if not self._jwk:
            logger.debug("JWKS requested but no key is configured - returning empty key set")
            return {"keys": []}

        public_fields = {"kty", "kid", "alg", "use", "n", "e"}
        public_jwk = {k: v for k, v in self._jwk.items() if k in public_fields}
        return {"keys": [public_jwk]}



"""JWE authentication middleware for Starlette/FastMCP."""
import logging
from contextvars import ContextVar
from dataclasses import dataclass
from typing import Any, Mapping, Sequence

from starlette.middleware.base import BaseHTTPMiddleware
from starlette.requests import Request
from starlette.responses import JSONResponse, Response

from .verifier import JWETokenVerifier

logger = logging.getLogger(__name__)

_user_context: ContextVar[dict] = ContextVar("user", default={})


@dataclass(frozen=True, slots=True)
class ScopeDefinition:
    """A JWT field that must be present for a request to be accepted."""

    scope: str

    def as_dict(self) -> dict[str, str]:
        return {"scope": self.scope}


class AuthUser(dict):
    """User claims with attribute access. Missing keys return None."""

    def __getattr__(self, name: str) -> Any:
        return self.get(name)


def get_user() -> AuthUser:
    """Get authenticated user from current request context."""
    return AuthUser(_user_context.get())


class JWKSAuthMiddleware(BaseHTTPMiddleware):
    """
    Middleware that decrypts JWE Bearer tokens and serves JWKS endpoint.

    Usage:
        app = mcp.http_app()
        app.add_middleware(JWKSAuthMiddleware, scopes=[...])
        uvicorn.run(app, host="0.0.0.0", port=8000)
    """

    def __init__(
        self,
        app,
        scopes: Sequence[ScopeDefinition | Mapping[str, str]] | None = None,
        verifier: JWETokenVerifier | None = None,
        jwks_path: str = "/.well-known/jwks.json",
        scopes_path: str = "/.well-known/fhswf-scopes",
    ):
        super().__init__(app)
        self.verifier = verifier or JWETokenVerifier()
        self.jwks_path = jwks_path
        self.scopes_path = scopes_path
        self.scopes = self._normalize_scopes(scopes)
        logger.info(
            "JWKSAuthMiddleware initialized (jwks_path=%s, scopes_path=%s, key_configured=%s, required_scopes=%d)",
            self.jwks_path,
            self.scopes_path,
            self.verifier.is_configured,
            len(self.scopes),
        )

    async def dispatch(self, request: Request, call_next) -> Response:
        if request.url.path == self.jwks_path:
            logger.debug("Serving JWKS endpoint: %s %s", request.method, request.url.path)
            return JSONResponse(self.verifier.get_jwks())
        if request.method == "GET" and request.url.path == self.scopes_path:
            logger.debug("Serving scopes endpoint: %s %s", request.method, request.url.path)
            return JSONResponse(
                {"scopes_supported": [scope.as_dict() for scope in self.scopes]},
                headers=self._cors_headers(),
            )

        auth_header = request.headers.get("authorization", "")
        if not auth_header.lower().startswith("bearer "):
            if auth_header:
                logger.warning(
                    "Unsupported Authorization scheme: %s %s (expected 'Bearer')",
                    request.method,
                    request.url.path,
                )
            else:
                logger.warning(
                    "Missing Bearer token: %s %s",
                    request.method,
                    request.url.path,
                )
            return self._invalid_token_response()

        claims = await self.verifier.verify_token(auth_header[7:]) or {}
        if not claims:
            logger.warning(
                "Bearer token provided but verification failed: %s %s (client=%s)",
                request.method,
                request.url.path,
                request.client.host if request.client else "unknown",
            )
            return self._invalid_token_response()

        missing_scopes = [scope.as_dict() for scope in self.scopes if scope.scope not in claims]
        if missing_scopes:
            logger.warning(
                "Rejecting request with missing scopes: %s %s (missing=%s)",
                request.method,
                request.url.path,
                ",".join(item["scope"] for item in missing_scopes),
            )
            return JSONResponse(
                {"error": "missing_scopes", "missing": missing_scopes},
                status_code=403,
            )

        logger.debug(
            "Authenticated request: %s %s (user=%s)",
            request.method,
            request.url.path,
            claims.get("email") or claims.get("sub") or claims.get("name", "unknown"),
        )

        token = _user_context.set(claims)
        try:
            return await call_next(request)
        finally:
            _user_context.reset(token)

    @staticmethod
    def _normalize_scopes(
        scopes: Sequence[ScopeDefinition | Mapping[str, str]] | None,
    ) -> tuple[ScopeDefinition, ...]:
        if not scopes:
            raise ValueError("JWKSAuthMiddleware requires at least one configured scope.")

        normalized: list[ScopeDefinition] = []
        seen: set[str] = set()
        for raw_scope in scopes:
            if isinstance(raw_scope, ScopeDefinition):
                scope = raw_scope
            elif isinstance(raw_scope, Mapping):
                scope_name = raw_scope.get("scope")
                if not isinstance(scope_name, str):
                    raise ValueError("Each configured scope must define a string 'scope' value.")
                scope = ScopeDefinition(scope=scope_name.strip())
            else:
                raise TypeError("Each configured scope must be a ScopeDefinition or mapping.")

            if not scope.scope:
                raise ValueError("Each configured scope must define a non-empty 'scope' value.")
            if scope.scope in seen:
                raise ValueError(f"Duplicate scope configured: {scope.scope}")

            seen.add(scope.scope)
            normalized.append(scope)

        return tuple(normalized)

    @staticmethod
    def _cors_headers() -> dict[str, str]:
        return {
            "Access-Control-Allow-Origin": "*",
            "Access-Control-Allow-Methods": "GET",
            "Access-Control-Allow-Headers": "*",
        }

    @staticmethod
    def _invalid_token_response() -> JSONResponse:
        return JSONResponse(
            {"error": "invalid_token"},
            status_code=401,
            headers={"WWW-Authenticate": "Bearer"},
        )



import argparse
import base64
import json
import sys
import types
from pathlib import Path

import mcp_auth_middleware.cli as cli


class FakeJWK(dict):
    @classmethod
    def generate(cls, kty: str, size: int):
        inst = cls()
        inst["kty"] = kty
        inst["size"] = size
        return inst

    def thumbprint(self) -> str:
        return "thumbprint"

    def export_private(self) -> str:
        return json.dumps({"kty": "RSA", "kid": self["kid"], "private": True})

    def export_public(self) -> str:
        return json.dumps({"kty": "RSA", "kid": self["kid"]})


def test_generate_keys_writes_jwks_files(monkeypatch, tmp_path) -> None:
    monkeypatch.setattr(cli, "jwk", types.SimpleNamespace(JWK=FakeJWK))

    private_path, public_path = cli.generate_keys(tmp_path)

    assert private_path.exists()
    assert public_path.exists()
    assert json.loads(private_path.read_text())["keys"][0] == {
        "kty": "RSA",
        "kid": "thumbprint",
        "private": True,
        "alg": "RSA-OAEP",
        "use": "enc",
    }
    assert json.loads(public_path.read_text())["keys"][0] == {
        "kty": "RSA",
        "kid": "thumbprint",
        "alg": "RSA-OAEP",
        "use": "enc",
    }


def test_secure_delete_uses_shred(monkeypatch, tmp_path) -> None:
    file_path = tmp_path / "secret.txt"
    file_path.write_text("secret")

    called = {}

    monkeypatch.setattr(cli.shutil, "which", lambda name: "/usr/bin/shred")

    def fake_run(cmd, check):
        called["cmd"] = cmd
        called["check"] = check

    monkeypatch.setattr(cli.subprocess, "run", fake_run)

    cli.secure_delete(file_path)

    assert called["cmd"][0] == "shred"
    assert called["check"] is True


def test_secure_delete_fallback_removes_file(monkeypatch, tmp_path) -> None:
    file_path = tmp_path / "secret.txt"
    file_path.write_text("secret")

    monkeypatch.setattr(cli.shutil, "which", lambda name: None)

    cli.secure_delete(file_path)

    assert not file_path.exists()


def test_clean_keys_removes_private_and_public(monkeypatch, tmp_path) -> None:
    private_key = tmp_path / "mcp-private.json"
    public_key = tmp_path / "mcp-public.json"
    private_key.write_text("private")
    public_key.write_text("public")

    deleted = []

    def fake_secure_delete(path: Path) -> None:
        deleted.append(path)
        path.unlink()

    monkeypatch.setattr(cli, "secure_delete", fake_secure_delete)

    removed = cli.clean_keys(tmp_path)

    assert str(private_key) in removed
    assert str(public_key) in removed
    assert deleted == [private_key]
    assert not private_key.exists()
    assert not public_key.exists()


def test_cmd_k8s_prints_secret(monkeypatch, tmp_path, capsys) -> None:
    private_key = tmp_path / "mcp-private.json"
    private_key.write_bytes(b"secret")

    def fake_generate_keys(output_dir: Path):
        return private_key, tmp_path / "mcp-public.json"

    monkeypatch.setattr(cli, "generate_keys", fake_generate_keys)

    args = argparse.Namespace(output=tmp_path, namespace="dev", secret_name="mcp-key")
    cli.cmd_k8s(args)

    captured = capsys.readouterr()
    expected_b64 = base64.b64encode(b"secret").decode("utf-8")

    assert "kind: Secret" in captured.out
    assert "name: mcp-key" in captured.out
    assert "namespace: dev" in captured.out
    assert expected_b64 in captured.out


def test_cmd_generate_prints_paths(monkeypatch, tmp_path, capsys) -> None:
    private_key = tmp_path / "private.json"
    public_key = tmp_path / "public.json"

    def fake_generate_keys(output_dir: Path):
        return private_key, public_key

    monkeypatch.setattr(cli, "generate_keys", fake_generate_keys)

    args = argparse.Namespace(output=tmp_path)
    cli.cmd_generate(args)

    output = capsys.readouterr().out

    assert str(private_key) in output
    assert str(public_key) in output
    assert "MCP_KEY_FILE_PATH" in output


def test_cmd_clean_prints_removed(monkeypatch, tmp_path, capsys) -> None:
    monkeypatch.setattr(cli, "clean_keys", lambda path: [str(path / "mcp-private.json")])

    args = argparse.Namespace(output=tmp_path)
    cli.cmd_clean(args)

    output = capsys.readouterr().out

    assert "Removed:" in output


def test_main_dispatches_generate(monkeypatch, tmp_path) -> None:
    called = {}

    def fake_cmd_generate(args):
        called["args"] = args

    monkeypatch.setattr(cli, "cmd_generate", fake_cmd_generate)
    monkeypatch.setattr(sys, "argv", ["mcp-secrets", "generate", "-o", str(tmp_path)])

    cli.main()

    assert called["args"].output == Path(str(tmp_path))
